// -*- C++ -*-
//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

#ifndef __GLIBCXX__
#error header can only be used when targeting libstdc++ or libsupc++
#endif

namespace std {

bad_alloc::bad_alloc() _NOEXCEPT
{
}

bad_array_new_length::bad_array_new_length() _NOEXCEPT
{
}

bad_cast::bad_cast() _NOEXCEPT
{
}

bad_typeid::bad_typeid() _NOEXCEPT
{
}

} // namespace std
