target triple = "wasm32-unknown-unknown"

; Function Attrs: norecurse nounwind readnone
define hidden i32 @archiveHidden() #0 {
entry:
    ret i32 0
}

; Function Attrs: norecurse nounwind readnone
define i32 @archiveDefault() #1 {
entry:
    ret i32 0
}
