//===- FrontendTiming.cpp - Implements Frontend timing utils  -------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//
//
// This file keps implementation of frontend timing utils.
//
//===----------------------------------------------------------------------===//

#include "clang/Frontend/Utils.h"

namespace clang {

bool FrontendTimesIsEnabled = false;

}
