_CLC_OVERLOAD _CLC_DECL event_t async_work_group_strided_copy(
  __CLC_DST_ADDR_SPACE __CLC_GENTYPE *dst,
  const __CLC_SRC_ADDR_SPACE __CLC_GENTYPE *src,
  size_t num_gentypes,
  size_t stride,
  event_t event);
