#ifndef ISL_ID_TYPE_H
#define ISL_ID_TYPE_H

#include <isl/list.h>

#if defined(__cplusplus)
extern "C" {
#endif

struct isl_id;
typedef struct isl_id isl_id;

ISL_DECLARE_LIST_TYPE(id)

#if defined(__cplusplus)
}
#endif

#endif
