target triple = "wasm32-unknown-unknown"

@g0 = global i32 1, align 4
@foo = global i32 1, align 4

define i32 @f1() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f2() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f3() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f4() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f5() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f6() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f7() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f8() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f9() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f10() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f11() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f12() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f13() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f14() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f15() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f16() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f17() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f18() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f19() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f20() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f21() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f22() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f23() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f24() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f25() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f26() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f27() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f28() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f29() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f30() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f31() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f32() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f33() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f34() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f35() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f36() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f37() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f38() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f39() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f40() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f41() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f42() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f43() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f44() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f45() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f46() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f47() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f48() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f49() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f50() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f51() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f52() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f53() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f54() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f55() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f56() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f57() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f58() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f59() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f60() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f61() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f62() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f63() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f64() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f65() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f66() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f67() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f68() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f69() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f70() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f71() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f72() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f73() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f74() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f75() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f76() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f77() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f78() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f79() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f80() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f81() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f82() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f83() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f84() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f85() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f86() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f87() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f88() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f89() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f90() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f91() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f92() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f93() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f94() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f95() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f96() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f97() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f98() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f99() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f100() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f101() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f102() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f103() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f104() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f105() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f106() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f107() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f108() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f109() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f110() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f111() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f112() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f113() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f114() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f115() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f116() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f117() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f118() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f119() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f120() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f121() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f122() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f123() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f124() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f125() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f126() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f127() {
entry:
  %0 = load i32, i32* @foo, align 4
  ret i32 %0
}

define i32 @f128() {
entry:
  %0 = load i32, i32* @g0, align 4
  ret i32 %0
}

define i32 @f129() {
entry:
  %0 = load i32, i32* @g0, align 4
  ret i32 %0
}
