.global far
.type far,%function
far = 0x201001c

.global too_far1
.type too_far1,%function
too_far1 = 0x2020014
.global too_far2
.type too_far2,%function
too_far2 = 0x2020020
.global too_far3
.type too_far3,%function
too_far3 = 0x202002c
