#ifndef ISL_IMATH_WRAP
#define ISL_IMATH_WRAP

#define MP_BADARG	ISL_MP_BADARG
#define MP_FALSE	ISL_MP_FALSE
#define MP_MEMORY	ISL_MP_MEMORY
#define MP_MINERR	ISL_MP_MINERR
#define MP_NEG	ISL_MP_NEG
#define MP_OK	ISL_MP_OK
#define MP_RANGE	ISL_MP_RANGE
#define MP_TRUE	ISL_MP_TRUE
#define MP_TRUNC	ISL_MP_TRUNC
#define MP_UNDEF	ISL_MP_UNDEF
#define MP_ZPOS	ISL_MP_ZPOS

#define impq_canonicalize	isl_impq_canonicalize
#define impq_clear	isl_impq_clear
#define impq_cmp	isl_impq_cmp
#define impq_denref	isl_impq_denref
#define impq_get_str	isl_impq_get_str
#define impq_init	isl_impq_init
#define impq_mul	isl_impq_mul
#define impq_numref	isl_impq_numref
#define impq_set	isl_impq_set
#define impq_set_str	isl_impq_set_str
#define impq_set_ui	isl_impq_set_ui
#define impq_sgn	isl_impq_sgn
#define impz_abs	isl_impz_abs
#define impz_add	isl_impz_add
#define impz_addmul	isl_impz_addmul
#define impz_add_ui	isl_impz_add_ui
#define impz_cdiv_q	isl_impz_cdiv_q
#define impz_clear	isl_impz_clear
#define impz_cmp	isl_impz_cmp
#define impz_cmpabs	isl_impz_cmpabs
#define impz_cmp_si	isl_impz_cmp_si
#define impz_divexact	isl_impz_divexact
#define impz_divexact_ui	isl_impz_divexact_ui
#define impz_divisible_p	isl_impz_divisible_p
#define impz_export	isl_impz_export
#define impz_fdiv_q	isl_impz_fdiv_q
#define impz_fdiv_r	isl_impz_fdiv_r
#define impz_gcd	isl_impz_gcd
#define impz_get_si	isl_impz_get_si
#define impz_get_str	isl_impz_get_str
#define impz_get_ui	isl_impz_get_ui
#define impz_import	isl_impz_import
#define impz_init	isl_impz_init
#define impz_lcm	isl_impz_lcm
#define impz_mul	isl_impz_mul
#define impz_mul_2exp	isl_impz_mul_2exp
#define impz_mul_ui	isl_impz_mul_ui
#define impz_neg	isl_impz_neg
#define impz_pow_ui	isl_impz_pow_ui
#define impz_set	isl_impz_set
#define impz_set_si	isl_impz_set_si
#define impz_set_str	isl_impz_set_str
#define impz_set_ui	isl_impz_set_ui
#define impz_sgn	isl_impz_sgn
#define impz_sizeinbase	isl_impz_sizeinbase
#define impz_sub	isl_impz_sub
#define impz_submul	isl_impz_submul
#define impz_sub_ui	isl_impz_sub_ui
#define impz_swap	isl_impz_swap
#define impz_tdiv_q	isl_impz_tdiv_q
#define mp_error_string	isl_mp_error_string
#define mp_int_abs	isl_mp_int_abs
#define mp_int_add	isl_mp_int_add
#define mp_int_add_value	isl_mp_int_add_value
#define mp_int_alloc	isl_mp_int_alloc
#define mp_int_binary_len	isl_mp_int_binary_len
#define mp_int_clear	isl_mp_int_clear
#define mp_int_compare	isl_mp_int_compare
#define mp_int_compare_unsigned	isl_mp_int_compare_unsigned
#define mp_int_compare_uvalue	isl_mp_int_compare_uvalue
#define mp_int_compare_value	isl_mp_int_compare_value
#define mp_int_compare_zero	isl_mp_int_compare_zero
#define mp_int_copy	isl_mp_int_copy
#define mp_int_count_bits	isl_mp_int_count_bits
#define mp_int_div	isl_mp_int_div
#define mp_int_divisible_value	isl_mp_int_divisible_value
#define mp_int_div_pow2	isl_mp_int_div_pow2
#define mp_int_div_value	isl_mp_int_div_value
#define mp_int_egcd	isl_mp_int_egcd
#define mp_int_expt	isl_mp_int_expt
#define mp_int_expt_full	isl_mp_int_expt_full
#define mp_int_exptmod	isl_mp_int_exptmod
#define mp_int_exptmod_bvalue	isl_mp_int_exptmod_bvalue
#define mp_int_exptmod_evalue	isl_mp_int_exptmod_evalue
#define mp_int_exptmod_known	isl_mp_int_exptmod_known
#define mp_int_expt_value	isl_mp_int_expt_value
#define mp_int_free	isl_mp_int_free
#define mp_int_gcd	isl_mp_int_gcd
#define mp_int_init	isl_mp_int_init
#define mp_int_init_copy	isl_mp_int_init_copy
#define mp_int_init_size	isl_mp_int_init_size
#define mp_int_init_uvalue	isl_mp_int_init_uvalue
#define mp_int_init_value	isl_mp_int_init_value
#define mp_int_invmod	isl_mp_int_invmod
#define mp_int_is_pow2	isl_mp_int_is_pow2
#define mp_int_lcm	isl_mp_int_lcm
#define mp_int_mod	isl_mp_int_mod
#define mp_int_mul	isl_mp_int_mul
#define mp_int_mul_pow2	isl_mp_int_mul_pow2
#define mp_int_mul_value	isl_mp_int_mul_value
#define mp_int_neg	isl_mp_int_neg
#define mp_int_read_binary	isl_mp_int_read_binary
#define mp_int_read_cstring	isl_mp_int_read_cstring
#define mp_int_read_string	isl_mp_int_read_string
#define mp_int_read_unsigned	isl_mp_int_read_unsigned
#define mp_int_redux_const	isl_mp_int_redux_const
#define mp_int_root	isl_mp_int_root
#define mp_int_set_uvalue	isl_mp_int_set_uvalue
#define mp_int_set_value	isl_mp_int_set_value
#define mp_int_sqr	isl_mp_int_sqr
#define mp_int_string_len	isl_mp_int_string_len
#define mp_int_sub	isl_mp_int_sub
#define mp_int_sub_value	isl_mp_int_sub_value
#define mp_int_swap	isl_mp_int_swap
#define mp_int_to_binary	isl_mp_int_to_binary
#define mp_int_to_int	isl_mp_int_to_int
#define mp_int_to_string	isl_mp_int_to_string
#define mp_int_to_uint	isl_mp_int_to_uint
#define mp_int_to_unsigned	isl_mp_int_to_unsigned
#define mp_int_unsigned_len	isl_mp_int_unsigned_len
#define mp_int_zero	isl_mp_int_zero
#define mp_rat_abs	isl_mp_rat_abs
#define mp_rat_add	isl_mp_rat_add
#define mp_rat_add_int	isl_mp_rat_add_int
#define mp_rat_alloc	isl_mp_rat_alloc
#define mp_rat_clear	isl_mp_rat_clear
#define mp_rat_compare	isl_mp_rat_compare
#define mp_rat_compare_unsigned	isl_mp_rat_compare_unsigned
#define mp_rat_compare_value	isl_mp_rat_compare_value
#define mp_rat_compare_zero	isl_mp_rat_compare_zero
#define mp_rat_copy	isl_mp_rat_copy
#define mp_rat_decimal_len	isl_mp_rat_decimal_len
#define mp_rat_denom	isl_mp_rat_denom
#define mp_rat_denom_ref	isl_mp_rat_denom_ref
#define mp_rat_div	isl_mp_rat_div
#define mp_rat_div_int	isl_mp_rat_div_int
#define mp_rat_expt	isl_mp_rat_expt
#define mp_rat_free	isl_mp_rat_free
#define mp_rat_init	isl_mp_rat_init
#define mp_rat_init_copy	isl_mp_rat_init_copy
#define mp_rat_init_size	isl_mp_rat_init_size
#define mp_rat_is_integer	isl_mp_rat_is_integer
#define mp_rat_mul	isl_mp_rat_mul
#define mp_rat_mul_int	isl_mp_rat_mul_int
#define mp_rat_neg	isl_mp_rat_neg
#define mp_rat_numer	isl_mp_rat_numer
#define mp_rat_numer_ref	isl_mp_rat_numer_ref
#define mp_rat_read_cdecimal	isl_mp_rat_read_cdecimal
#define mp_rat_read_cstring	isl_mp_rat_read_cstring
#define mp_rat_read_decimal	isl_mp_rat_read_decimal
#define mp_rat_read_string	isl_mp_rat_read_string
#define mp_rat_read_ustring	isl_mp_rat_read_ustring
#define mp_rat_recip	isl_mp_rat_recip
#define mp_rat_reduce	isl_mp_rat_reduce
#define mp_rat_set_uvalue	isl_mp_rat_set_uvalue
#define mp_rat_set_value	isl_mp_rat_set_value
#define mp_rat_sign	isl_mp_rat_sign
#define mp_rat_string_len	isl_mp_rat_string_len
#define mp_rat_sub	isl_mp_rat_sub
#define mp_rat_sub_int	isl_mp_rat_sub_int
#define mp_rat_to_decimal	isl_mp_rat_to_decimal
#define mp_rat_to_ints	isl_mp_rat_to_ints
#define mp_rat_to_string	isl_mp_rat_to_string
#define mp_rat_zero	isl_mp_rat_zero

#endif
