        .section        .idata$4
        .long           0
        .long           0
        .section        .idata$5
        .long           0
        .long           0

        .section        .idata$7
        .global         __test_lib_iname
__test_lib_iname:
       .asciz           "foo.dll"
