//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

 // test climits

#include <climits>

#include "test_macros.h"

#ifndef CHAR_BIT
#error CHAR_BIT not defined
#endif

#ifndef SCHAR_MIN
#error SCHAR_MIN not defined
#endif

#ifndef SCHAR_MAX
#error SCHAR_MAX not defined
#endif

#ifndef UCHAR_MAX
#error UCHAR_MAX not defined
#endif

#ifndef CHAR_MIN
#error CHAR_MIN not defined
#endif

#ifndef CHAR_MAX
#error CHAR_MAX not defined
#endif

#ifndef MB_LEN_MAX
#error MB_LEN_MAX not defined
#endif

#ifndef SHRT_MIN
#error SHRT_MIN not defined
#endif

#ifndef SHRT_MAX
#error SHRT_MAX not defined
#endif

#ifndef USHRT_MAX
#error USHRT_MAX not defined
#endif

#ifndef INT_MIN
#error INT_MIN not defined
#endif

#ifndef INT_MAX
#error INT_MAX not defined
#endif

#ifndef UINT_MAX
#error UINT_MAX not defined
#endif

#ifndef LONG_MIN
#error LONG_MIN not defined
#endif

#ifndef LONG_MAX
#error LONG_MAX not defined
#endif

#ifndef ULONG_MAX
#error ULONG_MAX not defined
#endif

#ifndef LLONG_MIN
#error LLONG_MIN not defined
#endif

#ifndef LLONG_MAX
#error LLONG_MAX not defined
#endif

#ifndef ULLONG_MAX
#error ULLONG_MAX not defined
#endif

int main(int, char**)
{

  return 0;
}
